/* Mptn -- regexp-like pattern matcher suited for morphological analysis
 * Copyright (C) 2000  George Bronnikov
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
  Working with character sets.
  (Turn them into colorsets?)

  $Id: mptn-chrset.h 1.2 Sun, 23 Jan 2000 20:56:51 -0600 goga $
*/
#ifndef MPTN_CHRSET_H
#define MPTN_CHRSET_H

#ifndef MPTN_TYPES_H
/* Need chr type, defined in mptn.h */
#  error "Can only include \"mptn-chrset.h\" after \"mptn-types.h\""
#endif

typedef void mptn_chrset_t;

/*
  The following assumes thar chr is char
*/

enum {
  CHR_NVAL = 256,	/* the number of possible chr values */
  BITS_PER_BYTE = 8,
  BITS_IN_INT = sizeof (gint) * BITS_PER_BYTE,
  INTS_IN_CHRSET = (CHR_NVAL + (BITS_IN_INT-1)) / BITS_IN_INT
};

/* Initialize an ampty chrset */
mptn_chrset_t *mptn_chrset_create ();

/* Destroy a chrset */
void mptn_chrset_destroy (mptn_chrset_t *chrset);

/* Add c to chrset */
void mptn_chrset_add (mptn_chrset_t *chrset, chr c);

/* Remove c from chrset */
void mptn_chrset_remove (mptn_chrset_t *chrset, chr c);

/* Is c in chrset? */
gboolean mptn_chrset_in (mptn_chrset_t *chrset, chr c);

/* Reverse a chrset -- works in place */
void mptn_chrset_reverse (mptn_chrset_t *chrset);

/* Copy a chrset */
mptn_chrset_t * mptn_chrset_copy (mptn_chrset_t *chrset);

#endif /* already included */
