/* Mptn -- regexp-like pattern matcher suited for morphological analysis
 * Copyright (C) 2000  George Bronnikov
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
  A test file -- so far, assembling Mptns manually.

  $Id: mptn-test.c 1.3 Sun, 23 Jan 2000 20:56:51 -0600 goga $
*/

#include <stdio.h>
#include <unistd.h>

#include "mptn.h"
#include "mptn-chrset.h"
#include "mptn-private.h"

enum {
  STRLEN = 256
};

int 
main ()
{
  chr buf[STRLEN+1];

  /* Set some predefined variable restrictions */
  {
    mptn_t *mptn;

    mptn = mptn_parse (NULL, "[aeiouy]");
    mptn_var_restrict (NULL, chra_quark_from_static_string ("v"), mptn);
    mptn_free (mptn);

    mptn = mptn_parse (NULL, "[bcdfghjklmnpqrstvwxz]");
    mptn_vname_restrict (NULL, "c", mptn);
    mptn_vname_restrict (NULL, "c1", mptn);
    mptn_vname_restrict (NULL, "c2", mptn);
    mptn_free (mptn);

    mptn_matcher_set (NULL, chra_quark_from_static_string ("str"),
		      &mptn_string_matcher_ops);
  }

  while (fgets (buf, STRLEN+1, stdin) != NULL)
    {
      chr *expr, *str, *rep;
      int ll;
      mptn_t *mptn, *mptn_rep;
      mptn_ectl_t *ectl;
      gboolean did_match;

      ll = strlen (buf) - 1;
      if (buf[ll] != '\n')
	g_error ("Input string too long");
      
      buf[ll] = '\0';

      if (!isatty (fileno (stdin)))
	{
	  /* Put out some tracing info */
	  puts ("-----------------");
	  puts (buf);
	}

      expr = buf;
      str = strchr (buf, '#');
      
      if (!str)
	g_error ("Input should have the form expr#str");
      
      *(str++) = '\0';

      /* We now have the expression to match */
      mptn = mptn_parse (NULL, expr);
      if (!mptn)
	g_error ("Failed to parse %s", expr);

      mptn_print (mptn, stdout, 0);
      
      rep = strchr (str, '#');
      if (rep)
	{
	  *(rep++) = '\0';
	  mptn_rep = mptn_parse (NULL, rep);

	  if (!mptn_rep)
	    g_error ("Failed to parse %s", rep);
	}
      else
	mptn_rep = NULL;
      
      did_match = FALSE;
      for (ectl = mptn_exec_start (mptn, str, NULL, NULL);
	   ectl;
	   ectl = mptn_exec_step (ectl))
	{
	  mptn_var_t *vars;
	  
	  did_match = TRUE;
	  puts ("Matched");
	  vars = mptn_ectl_vars (ectl);
	  
	  if (vars)
	    {
	      mptn_var_t *vi;

	      for(vi = vars; vi->vcode; vi++)
		if (vi->beg)
		  {
		    chr *val;
		    
		    val = chra_ndup (vi->beg, vi->end - vi->beg);
		    printf ("  %s: %s\n", 
			    chra_quark_to_string (vi->vcode), 
			    val);
		    g_free (val);
		  }
	    }

	  if (mptn_rep)
	    {
	      chr *rr;
	      
	      rr = mptn_subst (mptn_rep, vars, NULL);
	      
	      if (rr)
		printf ("Substitution: %s\n", rr);
	      else
		puts ("No substitution");
	      
	      g_free (rr);
	    }

	}
      
      if (!did_match)
	puts ("No match");
      
      mptn_free (mptn);
      mptn_free (mptn_rep);
    }
      
  return 0;
}

